package java.awt;

import java.util.Iterator;
import sun.awt.PeerEvent;
import java.beans.PropertyChangeEvent;
import sun.awt.AppContext;
import sun.awt.UngrabEvent;
import sun.util.CoreResourceBundleControl;
import java.util.Locale;
import sun.awt.AWTAccessor;
import sun.awt.SunToolkit;
import java.awt.font.TextAttribute;
import java.awt.im.InputMethodHighlight;
import java.util.ArrayList;
import java.util.EventListener;
import java.awt.event.AWTEventListenerProxy;
import java.beans.PropertyChangeListener;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.dnd.peer.DragSourceContextPeer;
import java.awt.dnd.DragGestureEvent;
import sun.security.util.SecurityConstants;
import java.util.MissingResourceException;
import java.awt.datatransfer.Clipboard;
import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;
import java.net.URL;
import sun.awt.HeadlessToolkit;
import java.util.StringTokenizer;
import java.security.AccessController;
import java.io.FileInputStream;
import java.security.PrivilegedAction;
import java.util.Properties;
import java.io.File;
import java.awt.image.ColorModel;
import java.awt.peer.FontPeer;
import sun.awt.NullComponentPeer;
import java.awt.peer.MouseInfoPeer;
import java.awt.peer.CheckboxMenuItemPeer;
import java.awt.peer.FileDialogPeer;
import java.awt.peer.MenuItemPeer;
import java.awt.peer.PopupMenuPeer;
import java.awt.peer.MenuPeer;
import java.awt.peer.MenuBarPeer;
import java.awt.peer.DialogPeer;
import java.awt.peer.WindowPeer;
import java.awt.peer.PanelPeer;
import java.awt.peer.CanvasPeer;
import java.awt.peer.FramePeer;
import java.awt.peer.ChoicePeer;
import java.awt.peer.TextAreaPeer;
import java.awt.peer.ScrollPanePeer;
import java.awt.peer.ScrollbarPeer;
import java.awt.peer.CheckboxPeer;
import java.awt.peer.ListPeer;
import java.awt.peer.LabelPeer;
import java.awt.peer.TextFieldPeer;
import java.awt.peer.ButtonPeer;
import java.awt.peer.DesktopPeer;
import java.util.HashMap;
import java.util.WeakHashMap;
import java.awt.event.AWTEventListener;
import java.beans.PropertyChangeSupport;
import java.util.Map;
import java.util.ResourceBundle;
import java.awt.peer.LightweightPeer;

public abstract class Toolkit
{
    private static LightweightPeer lightweightMarker;
    private static Toolkit toolkit;
    private static String atNames;
    private static ResourceBundle resources;
    private static ResourceBundle platformResources;
    private static boolean loaded;
    protected final Map<String, Object> desktopProperties;
    protected final PropertyChangeSupport desktopPropsSupport;
    private static final int LONG_BITS = 64;
    private int[] calls;
    private static volatile long enabledOnToolkitMask;
    private AWTEventListener eventListener;
    private WeakHashMap<AWTEventListener, Toolkit.SelectiveAWTEventListener> listener2SelectiveListener;
    
    public Toolkit() {
        this.desktopProperties = new HashMap();
        this.desktopPropsSupport = createPropertyChangeSupport(this);
        this.calls = new int[64];
        this.eventListener = null;
        this.listener2SelectiveListener = new WeakHashMap();
    }
    
    protected abstract DesktopPeer createDesktopPeer(Desktop p0) throws HeadlessException;
    
    protected abstract ButtonPeer createButton(Button p0) throws HeadlessException;
    
    protected abstract TextFieldPeer createTextField(TextField p0) throws HeadlessException;
    
    protected abstract LabelPeer createLabel(Label p0) throws HeadlessException;
    
    protected abstract ListPeer createList(List p0) throws HeadlessException;
    
    protected abstract CheckboxPeer createCheckbox(Checkbox p0) throws HeadlessException;
    
    protected abstract ScrollbarPeer createScrollbar(Scrollbar p0) throws HeadlessException;
    
    protected abstract ScrollPanePeer createScrollPane(ScrollPane p0) throws HeadlessException;
    
    protected abstract TextAreaPeer createTextArea(TextArea p0) throws HeadlessException;
    
    protected abstract ChoicePeer createChoice(Choice p0) throws HeadlessException;
    
    protected abstract FramePeer createFrame(Frame p0) throws HeadlessException;
    
    protected abstract CanvasPeer createCanvas(Canvas p0);
    
    protected abstract PanelPeer createPanel(Panel p0);
    
    protected abstract WindowPeer createWindow(Window p0) throws HeadlessException;
    
    protected abstract DialogPeer createDialog(Dialog p0) throws HeadlessException;
    
    protected abstract MenuBarPeer createMenuBar(MenuBar p0) throws HeadlessException;
    
    protected abstract MenuPeer createMenu(Menu p0) throws HeadlessException;
    
    protected abstract PopupMenuPeer createPopupMenu(PopupMenu p0) throws HeadlessException;
    
    protected abstract MenuItemPeer createMenuItem(MenuItem p0) throws HeadlessException;
    
    protected abstract FileDialogPeer createFileDialog(FileDialog p0) throws HeadlessException;
    
    protected abstract CheckboxMenuItemPeer createCheckboxMenuItem(CheckboxMenuItem p0) throws HeadlessException;
    
    protected MouseInfoPeer getMouseInfoPeer() {
        throw new UnsupportedOperationException("Not implemented");
    }
    
    protected LightweightPeer createComponent(Component component) {
        if (Toolkit.lightweightMarker == null) {
            Toolkit.lightweightMarker = new NullComponentPeer();
        }
        return Toolkit.lightweightMarker;
    }
    
    @Deprecated
    protected abstract FontPeer getFontPeer(String p0, int p1);
    
    protected void loadSystemColors(int[] array) throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
    }
    
    public void setDynamicLayout(boolean dynamicLayout) throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            getDefaultToolkit().setDynamicLayout(dynamicLayout);
        }
    }
    
    protected boolean isDynamicLayoutSet() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        return this != getDefaultToolkit() && getDefaultToolkit().isDynamicLayoutSet();
    }
    
    public boolean isDynamicLayoutActive() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        return this != getDefaultToolkit() && getDefaultToolkit().isDynamicLayoutActive();
    }
    
    public abstract Dimension getScreenSize() throws HeadlessException;
    
    public abstract int getScreenResolution() throws HeadlessException;
    
    public Insets getScreenInsets(GraphicsConfiguration graphicsConfiguration) throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().getScreenInsets(graphicsConfiguration);
        }
        return new Insets(0, 0, 0, 0);
    }
    
    public abstract ColorModel getColorModel() throws HeadlessException;
    
    @Deprecated
    public abstract String[] getFontList();
    
    @Deprecated
    public abstract FontMetrics getFontMetrics(Font p0);
    
    public abstract void sync();
    
    private static void initAssistiveTechnologies() {
        Toolkit.atNames = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            
            @Override
            public String run() {
                try {
                    FileInputStream inStream = new FileInputStream(new File(System.getProperty("user.home") + sep + ".accessibility.properties"));
                    properties.load(inStream);
                    inStream.close();
                }
                catch (Exception ex) {}
                if (properties.size() == 0) {
                    try {
                        FileInputStream inStream2 = new FileInputStream(new File(System.getProperty("java.home") + sep + "lib" + sep + "accessibility.properties"));
                        properties.load(inStream2);
                        inStream2.close();
                    }
                    catch (Exception ex2) {}
                }
                if (System.getProperty("javax.accessibility.screen_magnifier_present") == null) {
                    String property = properties.getProperty("screen_magnifier_present", null);
                    if (property != null) {
                        System.setProperty("javax.accessibility.screen_magnifier_present", property);
                    }
                }
                String value = System.getProperty("javax.accessibility.assistive_technologies");
                if (value == null) {
                    value = properties.getProperty("assistive_technologies", null);
                    if (value != null) {
                        System.setProperty("javax.accessibility.assistive_technologies", value);
                    }
                }
                return value;
            }
        });
    }
    
    private static void loadAssistiveTechnologies() {
        if (Toolkit.atNames != null) {
            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
            StringTokenizer stringTokenizer = new StringTokenizer(Toolkit.atNames, " ,");
            while (stringTokenizer.hasMoreTokens()) {
                String nextToken = stringTokenizer.nextToken();
                try {
                    Class<Object> clazz;
                    if (systemClassLoader != null) {
                        clazz = systemClassLoader.loadClass(nextToken);
                    }
                    else {
                        clazz = Class.forName(nextToken);
                    }
                    clazz.newInstance();
                }
                catch (ClassNotFoundException ex) {
                    throw new AWTError("Assistive Technology not found: " + nextToken);
                }
                catch (InstantiationException ex2) {
                    throw new AWTError("Could not instantiate Assistive Technology: " + nextToken);
                }
                catch (IllegalAccessException ex3) {
                    throw new AWTError("Could not access Assistive Technology: " + nextToken);
                }
                catch (Exception obj) {
                    throw new AWTError("Error trying to install Assistive Technology: " + nextToken + " " + obj);
                }
            }
        }
    }
    
    public static synchronized Toolkit getDefaultToolkit() {
        if (Toolkit.toolkit == null) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    Class<Object> clazz = null;
                    String property = System.getProperty("awt.toolkit");
                    try {
                        clazz = Class.forName(property);
                    }
                    catch (ClassNotFoundException ex) {
                        ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
                        if (systemClassLoader != null) {
                            try {
                                clazz = systemClassLoader.loadClass(property);
                            }
                            catch (ClassNotFoundException ex2) {
                                throw new AWTError("Toolkit not found: " + property);
                            }
                        }
                    }
                    try {
                        if (clazz != null) {
                            Toolkit.toolkit = (Toolkit)clazz.newInstance();
                            if (GraphicsEnvironment.isHeadless()) {
                                Toolkit.toolkit = new HeadlessToolkit(Toolkit.toolkit);
                            }
                        }
                    }
                    catch (InstantiationException ex3) {
                        throw new AWTError("Could not instantiate Toolkit: " + property);
                    }
                    catch (IllegalAccessException ex4) {
                        throw new AWTError("Could not access Toolkit: " + property);
                    }
                    return null;
                }
            });
            loadAssistiveTechnologies();
        }
        return Toolkit.toolkit;
    }
    
    public abstract Image getImage(String p0);
    
    public abstract Image getImage(URL p0);
    
    public abstract Image createImage(String p0);
    
    public abstract Image createImage(URL p0);
    
    public abstract boolean prepareImage(Image p0, int p1, int p2, ImageObserver p3);
    
    public abstract int checkImage(Image p0, int p1, int p2, ImageObserver p3);
    
    public abstract Image createImage(ImageProducer p0);
    
    public Image createImage(byte[] array) {
        return this.createImage(array, 0, array.length);
    }
    
    public abstract Image createImage(byte[] p0, int p1, int p2);
    
    public abstract PrintJob getPrintJob(Frame p0, String p1, Properties p2);
    
    public PrintJob getPrintJob(Frame frame, String s, JobAttributes jobAttributes, PageAttributes pageAttributes) {
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().getPrintJob(frame, s, jobAttributes, pageAttributes);
        }
        return this.getPrintJob(frame, s, null);
    }
    
    public abstract void beep();
    
    public abstract Clipboard getSystemClipboard() throws HeadlessException;
    
    public Clipboard getSystemSelection() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().getSystemSelection();
        }
        GraphicsEnvironment.checkHeadless();
        return null;
    }
    
    public int getMenuShortcutKeyMask() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        return 2;
    }
    
    public boolean getLockingKeyState(int n) throws UnsupportedOperationException {
        GraphicsEnvironment.checkHeadless();
        if (n != 20 && n != 144 && n != 145 && n != 262) {
            throw new IllegalArgumentException("invalid key for Toolkit.getLockingKeyState");
        }
        throw new UnsupportedOperationException("Toolkit.getLockingKeyState");
    }
    
    public void setLockingKeyState(int n, boolean b) throws UnsupportedOperationException {
        GraphicsEnvironment.checkHeadless();
        if (n != 20 && n != 144 && n != 145 && n != 262) {
            throw new IllegalArgumentException("invalid key for Toolkit.setLockingKeyState");
        }
        throw new UnsupportedOperationException("Toolkit.setLockingKeyState");
    }
    
    protected static Container getNativeContainer(Component component) {
        return component.getNativeContainer();
    }
    
    public Cursor createCustomCursor(Image image, Point point, String s) throws IndexOutOfBoundsException, HeadlessException {
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().createCustomCursor(image, point, s);
        }
        return new Cursor(0);
    }
    
    public Dimension getBestCursorSize(int n, int n2) throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().getBestCursorSize(n, n2);
        }
        return new Dimension(0, 0);
    }
    
    public int getMaximumCursorColors() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().getMaximumCursorColors();
        }
        return 0;
    }
    
    public boolean isFrameStateSupported(int n) throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        if (this != getDefaultToolkit()) {
            return getDefaultToolkit().isFrameStateSupported(n);
        }
        return n == 0;
    }
    
    private static void setPlatformResources(ResourceBundle platformResources) {
        Toolkit.platformResources = platformResources;
    }
    
    private static native void initIDs();
    
    static void loadLibraries() {
        if (!Toolkit.loaded) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    System.loadLibrary("awt");
                    return null;
                }
            });
            Toolkit.loaded = true;
        }
    }
    
    public static String getProperty(String s, String s2) {
        if (Toolkit.platformResources != null) {
            try {
                return Toolkit.platformResources.getString(s);
            }
            catch (MissingResourceException ex) {}
        }
        if (Toolkit.resources != null) {
            try {
                return Toolkit.resources.getString(s);
            }
            catch (MissingResourceException ex2) {}
            return s2;
        }
        return s2;
    }
    
    public final EventQueue getSystemEventQueue() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.CHECK_AWT_EVENTQUEUE_PERMISSION);
        }
        return this.getSystemEventQueueImpl();
    }
    
    protected abstract EventQueue getSystemEventQueueImpl();
    
    static EventQueue getEventQueue() {
        return getDefaultToolkit().getSystemEventQueueImpl();
    }
    
    public abstract DragSourceContextPeer createDragSourceContextPeer(DragGestureEvent p0) throws InvalidDnDOperationException;
    
    public <T extends DragGestureRecognizer> T createDragGestureRecognizer(Class<T> clazz, DragSource dragSource, Component component, int n, DragGestureListener dragGestureListener) {
        return null;
    }
    
    public final synchronized Object getDesktopProperty(String s) {
        if ((this instanceof HeadlessToolkit)) {
            return ((HeadlessToolkit)this).getUnderlyingToolkit().getDesktopProperty(s);
        }
        if (this.desktopProperties.isEmpty()) {
            this.initializeDesktopProperties();
        }
        if (s.equals("awt.dynamicLayoutSupported")) {
            return getDefaultToolkit().lazilyLoadDesktopProperty(s);
        }
        Object o = this.desktopProperties.get(s);
        if (o == null) {
            o = this.lazilyLoadDesktopProperty(s);
            if (o != null) {
                this.setDesktopProperty(s, o);
            }
        }
        if ((o instanceof RenderingHints)) {
            o = ((RenderingHints)o).clone();
            return o;
        }
        return o;
    }
    
    protected final void setDesktopProperty(String propertyName, Object newValue) {
        if ((this instanceof HeadlessToolkit)) {
            ((HeadlessToolkit)this).getUnderlyingToolkit().setDesktopProperty(propertyName, newValue);
            return;
        }
        Object value;
        synchronized (this) {
            value = this.desktopProperties.get(propertyName);
            this.desktopProperties.put(propertyName, newValue);
        }
        if (value != null || newValue != null) {
            this.desktopPropsSupport.firePropertyChange(propertyName, value, newValue);
        }
        return;
    }
    
    protected Object lazilyLoadDesktopProperty(String s) {
        return null;
    }
    
    protected void initializeDesktopProperties() {
    }
    
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.desktopPropsSupport.addPropertyChangeListener(propertyName, listener);
    }
    
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.desktopPropsSupport.removePropertyChangeListener(propertyName, listener);
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners() {
        return this.desktopPropsSupport.getPropertyChangeListeners();
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        return this.desktopPropsSupport.getPropertyChangeListeners(propertyName);
    }
    
    public boolean isAlwaysOnTopSupported() {
        return true;
    }
    
    public abstract boolean isModalityTypeSupported(Dialog.ModalityType p0);
    
    public abstract boolean isModalExclusionTypeSupported(Dialog.ModalExclusionType p0);
    
    private static AWTEventListener deProxyAWTEventListener(AWTEventListener awtEventListener) {
        if (awtEventListener == null) {
            return null;
        }
        if ((awtEventListener instanceof AWTEventListenerProxy)) {
            awtEventListener = (AWTEventListener)((AWTEventListenerProxy)awtEventListener).getListener();
            return awtEventListener;
        }
        return awtEventListener;
    }
    
    public void addAWTEventListener(AWTEventListener awtEventListener, long n) {
        AWTEventListener deProxyAWTEventListener = deProxyAWTEventListener(awtEventListener);
        if (deProxyAWTEventListener == null) {
            return;
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ALL_AWT_EVENTS_PERMISSION);
        }
        synchronized (this) {
            Toolkit.SelectiveAWTEventListener value = (Toolkit.SelectiveAWTEventListener)this.listener2SelectiveListener.get(deProxyAWTEventListener);
            if (value == null) {
                value = new Toolkit.SelectiveAWTEventListener(deProxyAWTEventListener, n);
                this.listener2SelectiveListener.put(deProxyAWTEventListener, value);
                this.eventListener = Toolkit.ToolkitEventMulticaster.add(this.eventListener, value);
            }
            value.orEventMasks(n);
            Toolkit.enabledOnToolkitMask |= n;
            for (int n2 = 0; n2 < 64 && n != 0L; n >>>= 1, ++n2) {
                if ((n & 0x1L) != 0x0L) {
                    int[] calls = this.calls;
                    int n3 = n2;
                    ++calls[n3];
                }
            }
        }
        return;
    }
    
    public void removeAWTEventListener(AWTEventListener awtEventListener) {
        AWTEventListener deProxyAWTEventListener = deProxyAWTEventListener(awtEventListener);
        if (awtEventListener == null) {
            return;
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ALL_AWT_EVENTS_PERMISSION);
        }
        synchronized (this) {
            Toolkit.SelectiveAWTEventListener selectiveAWTEventListener = (Toolkit.SelectiveAWTEventListener)this.listener2SelectiveListener.get(deProxyAWTEventListener);
            if (selectiveAWTEventListener != null) {
                this.listener2SelectiveListener.remove(deProxyAWTEventListener);
                int[] calls = selectiveAWTEventListener.getCalls();
                int i = 0;
                while (i < 64) {
                    int[] calls2 = this.calls;
                    int n = i;
                    calls2[n] -= calls[i];
                    assert this.calls[i] >= 0 : "Negative Listeners count";
                    if (this.calls[i] == 0) {
                        Toolkit.enabledOnToolkitMask &= ~(1L << i);
                    }
                    ++i;
                }
            }
            this.eventListener = Toolkit.ToolkitEventMulticaster.remove(this.eventListener, (selectiveAWTEventListener == null) ? deProxyAWTEventListener : selectiveAWTEventListener);
        }
        return;
    }
    
    static boolean enabledOnToolkit(long n) {
        return (Toolkit.enabledOnToolkitMask & n) != 0x0L;
    }
    
    synchronized int countAWTEventListeners(long n) {
        int n2;
        for (n2 = 0; n != 0L; n >>>= 1, ++n2) {}
        --n2;
        return this.calls[n2];
    }
    
    public AWTEventListener[] getAWTEventListeners() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ALL_AWT_EVENTS_PERMISSION);
        }
        synchronized (this) {
            EventListener[] listeners = AWTEventMulticaster.getListeners(this.eventListener, AWTEventListener.class);
            AWTEventListener[] array = new AWTEventListener[listeners.length];
            for (int i = 0; i < listeners.length; ++i) {
                Toolkit.SelectiveAWTEventListener selectiveAWTEventListener = (Toolkit.SelectiveAWTEventListener)listeners[i];
                array[i] = new AWTEventListenerProxy(selectiveAWTEventListener.getEventMask(), selectiveAWTEventListener.getListener());
            }
            return array;
        }
    }
    
    public AWTEventListener[] getAWTEventListeners(long n) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ALL_AWT_EVENTS_PERMISSION);
        }
        synchronized (this) {
            EventListener[] listeners = AWTEventMulticaster.getListeners(this.eventListener, AWTEventListener.class);
            ArrayList<AWTEventListenerProxy> list = new ArrayList(listeners.length);
            for (int i = 0; i < listeners.length; ++i) {
                Toolkit.SelectiveAWTEventListener selectiveAWTEventListener = (Toolkit.SelectiveAWTEventListener)listeners[i];
                if ((selectiveAWTEventListener.getEventMask() & n) == n) {
                    list.add(new AWTEventListenerProxy(selectiveAWTEventListener.getEventMask(), selectiveAWTEventListener.getListener()));
                }
            }
            return (AWTEventListener[])list.toArray(new AWTEventListener[0]);
        }
    }
    
    void notifyAWTEventListeners(AWTEvent awtEvent) {
        if ((this instanceof HeadlessToolkit)) {
            ((HeadlessToolkit)this).getUnderlyingToolkit().notifyAWTEventListeners(awtEvent);
            return;
        }
        AWTEventListener eventListener = this.eventListener;
        if (eventListener != null) {
            eventListener.eventDispatched(awtEvent);
        }
        return;
    }
    
    public abstract Map<TextAttribute, ?> mapInputMethodHighlight(InputMethodHighlight p0) throws HeadlessException;
    
    private static PropertyChangeSupport createPropertyChangeSupport(Toolkit sourceBean) {
        if (sourceBean instanceof SunToolkit || sourceBean instanceof HeadlessToolkit) {
            return new Toolkit.DesktopPropertyChangeSupport(sourceBean);
        }
        return new PropertyChangeSupport(sourceBean);
    }
    
    public boolean areExtraMouseButtonsEnabled() throws HeadlessException {
        GraphicsEnvironment.checkHeadless();
        return getDefaultToolkit().areExtraMouseButtonsEnabled();
    }
    
    static void access$100(ResourceBundle platformResources) {
        setPlatformResources(platformResources);
    }
    
    static {
        Toolkit.loaded = false;
        AWTAccessor.setToolkitAccessor(new AWTAccessor.ToolkitAccessor() {
            @Override
            public void setPlatformResources(ResourceBundle resourceBundle) {
                Toolkit.access$100(resourceBundle);
            }
        });
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                try {
                    Toolkit.resources = ResourceBundle.getBundle("sun.awt.resources.awt", Locale.getDefault(), ClassLoader.getSystemClassLoader(), CoreResourceBundleControl.getRBControlInstance());
                }
                catch (MissingResourceException ex) {}
                return null;
            }
        });
        loadLibraries();
        initAssistiveTechnologies();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
    }
    
    private static class ToolkitEventMulticaster extends AWTEventMulticaster implements AWTEventListener
    {
        ToolkitEventMulticaster(AWTEventListener awtEventListener, AWTEventListener awtEventListener2) {
            super(awtEventListener, awtEventListener2);
        }
        
        static AWTEventListener add(AWTEventListener awtEventListener, AWTEventListener awtEventListener2) {
            if (awtEventListener == null) {
                return awtEventListener2;
            }
            if (awtEventListener2 == null) {
                return awtEventListener;
            }
            return new ToolkitEventMulticaster(awtEventListener, awtEventListener2);
        }
        
        static AWTEventListener remove(AWTEventListener awtEventListener, AWTEventListener awtEventListener2) {
            return (AWTEventListener)AWTEventMulticaster.removeInternal(awtEventListener, awtEventListener2);
        }
        
        @Override
        protected EventListener remove(EventListener eventListener) {
            if (eventListener == this.a) {
                return this.b;
            }
            if (eventListener == this.b) {
                return this.a;
            }
            AWTEventListener awtEventListener = (AWTEventListener)AWTEventMulticaster.removeInternal(this.a, eventListener);
            AWTEventListener awtEventListener2 = (AWTEventListener)AWTEventMulticaster.removeInternal(this.b, eventListener);
            if (awtEventListener == this.a && awtEventListener2 == this.b) {
                return this;
            }
            return add(awtEventListener, awtEventListener2);
        }
        
        @Override
        public void eventDispatched(AWTEvent awtEvent) {
            ((AWTEventListener)this.a).eventDispatched(awtEvent);
            ((AWTEventListener)this.b).eventDispatched(awtEvent);
        }
    }
    
    private class SelectiveAWTEventListener implements AWTEventListener
    {
        AWTEventListener listener;
        private long eventMask;
        int[] calls;
        
        public AWTEventListener getListener() {
            return this.listener;
        }
        
        public long getEventMask() {
            return this.eventMask;
        }
        
        public int[] getCalls() {
            return this.calls;
        }
        
        public void orEventMasks(long n) {
            this.eventMask |= n;
            for (int n2 = 0; n2 < 64 && n != 0L; n >>>= 1, ++n2) {
                if ((n & 0x1L) != 0x0L) {
                    int[] calls = this.calls;
                    int n3 = n2;
                    ++calls[n3];
                }
            }
        }
        
        SelectiveAWTEventListener(AWTEventListener listener, long eventMask) {
            this.calls = new int[64];
            this.listener = listener;
            this.eventMask = eventMask;
        }
        
        @Override
        public void eventDispatched(AWTEvent awtEvent) {
            long n;
            if (((n = (this.eventMask & 0x1L)) != 0L && awtEvent.id >= 100 && awtEvent.id <= 103) || ((n = (this.eventMask & 0x2L)) != 0L && awtEvent.id >= 300 && awtEvent.id <= 301) || ((n = (this.eventMask & 0x4L)) != 0L && awtEvent.id >= 1004 && awtEvent.id <= 1005) || ((n = (this.eventMask & 0x8L)) != 0L && awtEvent.id >= 400 && awtEvent.id <= 402) || ((n = (this.eventMask & 0x20000L)) != 0L && awtEvent.id == 507) || ((n = (this.eventMask & 0x20L)) != 0L && (awtEvent.id == 503 || awtEvent.id == 506)) || ((n = (this.eventMask & 0x10L)) != 0L && awtEvent.id != 503 && awtEvent.id != 506 && awtEvent.id != 507 && awtEvent.id >= 500 && awtEvent.id <= 507) || ((n = (this.eventMask & 0x40L)) != 0L && awtEvent.id >= 200 && awtEvent.id <= 209) || ((n = (this.eventMask & 0x80L)) != 0L && awtEvent.id >= 1001 && awtEvent.id <= 1001) || ((n = (this.eventMask & 0x100L)) != 0L && awtEvent.id >= 601 && awtEvent.id <= 601) || ((n = (this.eventMask & 0x200L)) != 0L && awtEvent.id >= 701 && awtEvent.id <= 701) || ((n = (this.eventMask & 0x400L)) != 0L && awtEvent.id >= 900 && awtEvent.id <= 900) || ((n = (this.eventMask & 0x800L)) != 0L && awtEvent.id >= 1100 && awtEvent.id <= 1101) || ((n = (this.eventMask & 0x2000L)) != 0L && awtEvent.id >= 800 && awtEvent.id <= 801) || ((n = (this.eventMask & 0x4000L)) != 0L && awtEvent.id >= 1200 && awtEvent.id <= 1200) || ((n = (this.eventMask & 0x8000L)) != 0L && awtEvent.id == 1400) || ((n = (this.eventMask & 0x10000L)) != 0L && (awtEvent.id == 1401 || awtEvent.id == 1402)) || ((n = (this.eventMask & 0x40000L)) != 0L && awtEvent.id == 209) || ((n = (this.eventMask & 0x80000L)) != 0L && (awtEvent.id == 207 || awtEvent.id == 208)) || ((n = (this.eventMask & 0xFFFFFFFF80000000L)) != 0L && awtEvent instanceof UngrabEvent)) {
                int n2;
                for (n2 = 0, ; n != 0L; n >>>= 1, ++n2) {}
                --n2;
                for (int i = 0; i < this.calls[n2]; ++i) {
                    this.listener.eventDispatched(awtEvent);
                }
            }
        }
    }
    
    private static class DesktopPropertyChangeSupport extends PropertyChangeSupport
    {
        private static final StringBuilder PROP_CHANGE_SUPPORT_KEY;
        private final Object source;
        
        public DesktopPropertyChangeSupport(Object o) {
            super(o);
            this.source = o;
        }
        
        @Override
        public synchronized void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
            PropertyChangeSupport value = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null == value) {
                value = new PropertyChangeSupport(this.source);
                AppContext.getAppContext().put(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY, value);
            }
            value.addPropertyChangeListener(propertyName, listener);
        }
        
        @Override
        public synchronized void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
            PropertyChangeSupport propertyChangeSupport = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null != propertyChangeSupport) {
                propertyChangeSupport.removePropertyChangeListener(propertyName, listener);
            }
        }
        
        @Override
        public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
            PropertyChangeSupport propertyChangeSupport = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null != propertyChangeSupport) {
                return propertyChangeSupport.getPropertyChangeListeners();
            }
            return new PropertyChangeListener[0];
        }
        
        @Override
        public synchronized PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
            PropertyChangeSupport propertyChangeSupport = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null != propertyChangeSupport) {
                return propertyChangeSupport.getPropertyChangeListeners(propertyName);
            }
            return new PropertyChangeListener[0];
        }
        
        @Override
        public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
            PropertyChangeSupport value = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null == value) {
                value = new PropertyChangeSupport(this.source);
                AppContext.getAppContext().put(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY, value);
            }
            value.addPropertyChangeListener(listener);
        }
        
        @Override
        public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
            PropertyChangeSupport propertyChangeSupport = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
            if (null != propertyChangeSupport) {
                propertyChangeSupport.removePropertyChangeListener(listener);
            }
        }
        
        @Override
        public void firePropertyChange(PropertyChangeEvent propertyChangeEvent) {
            Object oldValue = propertyChangeEvent.getOldValue();
            Object newValue = propertyChangeEvent.getNewValue();
            propertyChangeEvent.getPropertyName();
            if (oldValue != null && newValue != null && oldValue.equals(newValue)) {
                return;
            }
            Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    PropertyChangeSupport propertyChangeSupport = (PropertyChangeSupport)AppContext.getAppContext().get(Toolkit.DesktopPropertyChangeSupport.PROP_CHANGE_SUPPORT_KEY);
                    if (null != propertyChangeSupport) {
                        propertyChangeSupport.firePropertyChange(propertyChangeEvent);
                    }
                }
            };
            AppContext appContext = AppContext.getAppContext();
            for (AppContext appContext2 : AppContext.getAppContexts()) {
                if (null == appContext2) {
                    continue;
                }
                if (appContext2.isDisposed()) {
                    continue;
                }
                if (appContext == appContext2) {
                    runnable.run();
                }
                else {
                    SunToolkit.postEvent(appContext2, new PeerEvent(this.source, runnable, 2L));
                }
            }
            return;
        }
        
        static {
            PROP_CHANGE_SUPPORT_KEY = new StringBuilder("desktop property change support key");
        }
    }
}
